from unittest import TestCase

from lotion.block.code import Code


class TestCode(TestCase):
    def setUp(self) -> None:
        return super().setUp()

    def test(self):
        # Given
        input = {
            "object": "block",
            "id": "83dc84f8-2bee-4ed4-9ea2-df66eafe6e72",
            "parent": {"type": "page_id", "page_id": "c560bb27-1b52-4d52-83a6-4b8e7615ea9d"},
            "created_time": "2024-03-08T06:43:00.000Z",
            "last_edited_time": "2024-03-08T06:43:00.000Z",
            "created_by": {"object": "user", "id": "6f877603-07b9-4467-b811-31706eeb640c"},
            "last_edited_by": {"object": "user", "id": "6f877603-07b9-4467-b811-31706eeb640c"},
            "has_children": False,
            "archived": False,
            "type": "code",
            "code": {
                "caption": [],
                "rich_text": [
                    {
                        "type": "text",
                        "text": {"content": 'setcookie("user", "alex", time()+3600);', "link": None},
                        "annotations": {
                            "bold": False,
                            "italic": False,
                            "strikethrough": False,
                            "underline": False,
                            "code": False,
                            "color": "default",
                        },
                        "plain_text": 'setcookie("user", "alex", time()+3600);',
                        "href": None,
                    }
                ],
                "language": "php",
            },
        }

        # When
        actual = Code.of(input)

        # Then
        self.assertEqual(actual.type, "code")
        self.assertEqual(actual.language, "php")
        self.assertEqual(actual.rich_text.to_plain_text(), 'setcookie("user", "alex", time()+3600);')
        expected_slack_text = """```php
setcookie("user", "alex", time()+3600);
```"""
        self.assertEqual(actual.to_slack_text(), expected_slack_text)
        expected_sub_dict = {
            "rich_text": [
                {
                    "plain_text": 'setcookie("user", "alex", time()+3600);',
                    "annotations": {
                        "bold": False,
                        "italic": False,
                        "strikethrough": False,
                        "underline": False,
                        "code": False,
                        "color": "default",
                    },
                    "href": None,
                    "text": {"content": 'setcookie("user", "alex", time()+3600);', "link": None},
                    "type": "text",
                }
            ],
            "language": "php",
        }
        actual_sub_dict = actual.to_dict_sub()
        self.assertEqual(actual_sub_dict["language"], expected_sub_dict["language"])
        self.assertEqual(actual_sub_dict["rich_text"][0]["plain_text"], expected_sub_dict["rich_text"][0]["plain_text"])



from unittest import TestCase

import pytest

from lotion.properties.email import Email
from lotion.properties.phone_number import PhoneNumber
from test.test_lotion.lotion_utils import create_empty_page, remove_page, update_page


@pytest.mark.api()
class TestApiEmailAndPhone(TestCase):
    DATABASE_ID = "15d6567a3bbf80bf94b8f2d46d195724"

    def setUp(self) -> None:
        self.page = create_empty_page(database_id=self.DATABASE_ID)
        return super().setUp()

    def tearDown(self) -> None:
        remove_page(page_id=self.page.page_id)
        return super().setUp()

    def test_メールを変更する(self):
        email_prop = Email.from_email(name="メール", email="sample@example.com")
        actual = update_page(page=self.page, property=email_prop)
        self.assertEqual(actual.get_email(name="メール").value, "sample@example.com")

        email_empty_prop = Email.empty(name="メール")
        actual = update_page(page=self.page, property=email_empty_prop)
        self.assertEqual(actual.get_email(name="メール").value, "")

    def test_電話番号を変更する(self):
        phone_number_prop = PhoneNumber.create(name="電話", phone_number="090-1234-5678")
        actual = update_page(page=self.page, property=phone_number_prop)
        self.assertEqual(actual.get_phone_number(name="電話").value, "090-1234-5678")

        phone_number_empty_prop = PhoneNumber.empty(name="電話")
        actual = update_page(page=self.page, property=phone_number_empty_prop)
        self.assertEqual(actual.get_phone_number(name="電話").value, "")



from dataclasses import dataclass

from lotion.properties.property import Property


@dataclass
class PhoneNumber(Property):
    """PhoneNumber class

    ex.
    {'id': 'FCsG', 'type': 'phone_number', 'phone_number': '03-1234-5678'}
    """

    value: str

    def __init__(
        self,
        name: str,
        value: str = "",
        id: str | None = None,  # noqa: A002
    ) -> None:
        self.name = name
        self.value = value
        self.id = id

    @staticmethod
    def of(key: str, param: dict) -> "PhoneNumber":
        value = param.get("phone_number")
        if value is not None and not isinstance(value, str):
            raise ValueError(f"phone_number must be str, but got {type(value)}")
        return PhoneNumber(id=param["id"], name=key, value=value or "")

    @staticmethod
    def empty(name: str) -> "PhoneNumber":
        return PhoneNumber(name=name)

    @staticmethod
    def create(name: str, phone_number: str) -> "PhoneNumber":
        return PhoneNumber(name=name, value=phone_number)

    @property
    def type(self) -> str:
        return "phone_number"

    def __dict__(self) -> dict:
        result = {
            "type": self.type,
            "phone_number": self.value if self.value != "" else None,
        }
        if self.id is not None:
            result["id"] = self.id
        return {
            self.name: result,
        }

    def value_for_filter(self) -> str:
        raise NotImplementedError



from lotion.block import Block
from lotion.block.rich_text import RichText


class Paragraph(Block):
    rich_text: RichText
    color: str | None = None

    def __init__(
        self,
        rich_text: RichText,
        color: str | None = None,
        id: str | None = None,
        archived: bool | None = None,
        created_time: str | None = None,
        last_edited_time: str | None = None,
        has_children: bool | None = None,
        parent: dict | None = None,
    ):
        super().__init__(id, archived, created_time, last_edited_time, has_children, parent)
        self.rich_text = rich_text
        self.color = color

    @staticmethod
    def of(block: dict) -> "Paragraph":
        paragraph = block["paragraph"]
        rich_text = RichText.from_entity(paragraph["rich_text"])
        return Paragraph(
            rich_text=rich_text,
            id=block["id"],
            archived=block["archived"],
            created_time=block["created_time"],
            last_edited_time=block["last_edited_time"],
            has_children=block["has_children"],
            parent=block["parent"],
            color=paragraph["color"],
        )

    @staticmethod
    def from_rich_text(rich_text: RichText) -> "Paragraph":
        return Paragraph(rich_text=rich_text)

    @staticmethod
    def from_plain_text(text: str) -> "Paragraph":
        rich_text = RichText.from_plain_text(text)
        return Paragraph(rich_text=rich_text)

    @property
    def type(self) -> str:
        return "paragraph"

    def to_dict_sub(self) -> dict:
        result = {
            "rich_text": self.rich_text.to_dict(),
        }
        if self.color is not None:
            result["color"] = self.color
        return result

    def to_slack_text(self) -> str:
        return self.rich_text.to_slack_text()



import sys
from unittest import TestCase

from lotion.page.page_id import PageId

sys.path.append("notion_api")

# クラス名の一致をチェックするロジックがあるので、完全に合わせておく
from lotion.properties.title import Title


class TestTitle(TestCase):
    def test_シンプルなテキスト(self) -> None:
        input = "dummy"
        actual = Title.from_plain_text(text=input)

        # Then
        self.assertEqual(actual.text, input)

    def test_テキストとページメンションを合わせて利用(self) -> None:
        input = "dummy"
        page_id = PageId.dummy()
        page_title = "Mentioned Page"
        actual = Title.from_mentioned_page(
            mentioned_page_id=page_id, mentioned_page_title=page_title, prefix=input
        )

        # Then
        self.assertEqual(actual.text, "dummyMentioned Page")



from unittest import TestCase

from lotion.properties.created_by import CreatedBy


# クラス名の一致をチェックするロジックがあるので、完全に合わせておく


class TestCreatedBy(TestCase):
    def test_of(self) -> None:
        key = "作成者"
        input = {
            "id": "VbPv",
            "type": "created_by",
            "created_by": {
                "object": "user",
                "id": "510806db-4772-4f42-b4b6-6f81b6e8b788",
                "name": "daily-api",
                "avatar_url": None,
                "type": "bot",
                "bot": {},
            },
        }
        actual = CreatedBy.of(key, input)

        # Then
        self.assertEqual(actual.id, "VbPv")



from dataclasses import dataclass

from lotion.properties.property import Property


@dataclass
class Status(Property):
    status_id: str | None
    status_name: str
    status_color: str | None
    type: str = "status"

    def __init__(self,
                 name: str,
                 status_name: str,
                 id: str | None = None,
                 status_id: str | None = None,
                 status_color: str | None = None):
        self.name = name
        self.status_name = status_name
        self.id = id
        self.status_id = status_id
        self.status_color = status_color

    @ staticmethod
    def of(name: str, param: dict) -> "Status":
        return Status(
            name=name,
            status_name=param["status"]["name"],
            id=param["id"],
            status_id=param["status"]["id"],
            status_color=param["status"]["color"],
        )

    @staticmethod
    def from_status_name(name: str, status_name: str) -> "Status":
        return Status(
            name=name,
            status_name=status_name,
        )

    def is_today(self) -> bool:
        return self.status_name == "Today"

    def __dict__(self):
        result = {
            "type": self.type,
            "status": {
                "name": self.status_name,
            },
        }
        if self.status_id is not None:
            result["status"]["id"] = self.status_id
        if self.status_color is not None:
            result["status"]["color"] = self.status_color
        return {self.name: result}

    def value_for_filter(self) -> str:
        return self.status_name



from unittest import TestCase

import pytest

from lotion import Lotion
from lotion.properties.number import Number


@pytest.mark.api()
class TestApiFormula(TestCase):
    DATABASE_ID = "15d6567a3bbf80fe8855d0383a7eb7dd"

    def setUp(self) -> None:
        self.suite = Lotion.get_instance()

    def test_数式の入ったページを取得する(self):
        # When
        actual = self.suite.retrieve_page(page_id="15d6567a3bbf803d9c3eeebb80b24d89")

        # Then
        actual_string = actual.get_formula(name="数式_文字列").value
        self.assertEqual(actual_string, "100円です")
        actual_number = actual.get_formula(name="数式_数値").value
        self.assertEqual(actual_number, 50)

    def test_数式の入ったページを更新する(self):
        # When
        properties = [
            Number.from_num(name="数値", value=200),
        ]
        new_page = self.suite.create_page_in_database(
            database_id=self.DATABASE_ID,
            properties=properties,
        )
        properties = new_page.properties.append_property(Number.from_num(name="数値", value=300))
        self.suite.update_page(
            page_id=new_page.page_id.value,
            properties=properties.values,
        )
        actual = self.suite.retrieve_page(page_id=new_page.page_id.value)



from datetime import date, datetime, timedelta, timezone
from enum import Enum

JST = timezone(timedelta(hours=+9), "JST")

LENGTH_DATE = 10  # "YYYY-MM-DD"


class DateType(Enum):
    DATE = "date"
    DATETIME = "datetime"
    NONE = "none"

    @staticmethod
    def get_datetype(value: date | datetime | None) -> "DateType":
        if value is None:
            return DateType.NONE
        try:
            # それぞれ変換できることを確認してから返却する
            if len(value) == LENGTH_DATE:
                date.fromisoformat(value)
                return DateType.DATE
            datetime.fromisoformat(value)
            return DateType.DATETIME
        except ValueError:
            return DateType.NONE


def jst_now() -> datetime:
    return datetime.now(JST)


def jst_today_datetime() -> datetime:
    return jst_now().replace(hour=0, minute=0, second=0, microsecond=0)


def jst_today(is_previous_day_until_2am: bool | None = None) -> date:
    """Return today's date in JST
    Args:
        isPreviousDayUntil2AM (bool): If True, return the previous day until 2AM. Defaults to None.
    Returns:
        date: today's date in JST
    """
    now = jst_now()
    if is_previous_day_until_2am:
        return now.date() - timedelta(days=1) if now.hour < 2 else now.date()
    return now.date()


def jst_tommorow() -> datetime:
    return jst_today_datetime() + timedelta(days=1)


def convert_to_date_or_datetime(
    value: str | None, cls: type | None = None
) -> date | datetime | None:
    date_type = DateType.get_datetype(value)
    match date_type:
        case DateType.DATE:
            tmp_date = date.fromisoformat(value)
            return (
                datetime(tmp_date.year, tmp_date.month, tmp_date.day, tzinfo=JST)
                if cls == datetime
                else tmp_date
            )
        case DateType.DATETIME:
            tmp_datetime = datetime.fromisoformat(value)
            return (
                tmp_datetime.date()
                if cls == date or (tmp_datetime.hour == 0 and tmp_datetime.minute == 0)
                else tmp_datetime
            )
        case DateType.NONE:
            return None



from dataclasses import dataclass
from datetime import date as DateObject
from datetime import datetime as DatetimeObject
from datetime import timedelta
from enum import Enum


class TimeKind(Enum):
    CREATED_TIME = "created_time"
    LAST_EDITED_TIME = "last_edited_time"


@dataclass
class NotionDatetime:
    value: DatetimeObject
    kind: TimeKind

    @classmethod
    def created_time(cls, value: str) -> "NotionDatetime":
        datetime = DatetimeObject.fromisoformat(value[:-1])
        datetime += timedelta(hours=9)
        return cls(
            value=datetime,
            kind=TimeKind.CREATED_TIME,
        )

    @classmethod
    def last_edited_time(cls, value: str) -> "NotionDatetime":
        datetime = cls.__to_datetime(value)
        return cls(
            value=datetime,
            kind=TimeKind.LAST_EDITED_TIME,
        )

    @classmethod
    def __to_datetime(cls, value: str) -> DatetimeObject:
        try:
            datetime = DatetimeObject.fromisoformat(value[:-1])
            # FIXME: 日本以外にも対応したい
            datetime += timedelta(hours=9)
            return datetime
        except Exception as e:
            raise Exception(f"Invalid datetime format: {value} {e}")

    def is_between(self, start: DatetimeObject, end: DatetimeObject) -> bool:
        return start.timestamp() <= self.value.timestamp() <= end.timestamp()


    @property
    def date(self) -> DateObject:
        return self.value.date()

    def __dict__(self):
        return {self.kind.value: self.value.isoformat() + "Z"}

    def value_for_filter(self) -> str:
        raise NotImplementedError



from unittest import TestCase

import pytest

from lotion import Lotion


@pytest.mark.api()
class TestApiUniqueId(TestCase):
    DATABASE_ID = "15d6567a3bbf80f9ac95e5153875b2c3"

    def setUp(self) -> None:
        self.suite = Lotion.get_instance()

    def test_ユニークIDを取得する(self):
        # When
        page_id = "15d6567a3bbf8078bd6dfe1dc7688131"
        page = self.suite.retrieve_page(page_id=page_id)
        actual = page.get_unique_id(name="ID")

        # Then
        self.assertEqual(actual.number, 1)
        self.assertEqual(actual.prefix, "TEST")

        # Then: 更新が可能なこともついでに確かめる
        self.suite.update_page(page_id=page_id, properties=page.properties.values)

    def test_ユニークIDをもとにページを取得する(self):
        # When
        actual = self.suite.find_page_by_unique_id(database_id=self.DATABASE_ID, unique_id=1)

        # Then
        self.assertIsNotNone(actual)



from unittest import TestCase

import pytest

from test.test_lotion.lotion_utils import create_empty_page, remove_page, update_page


@pytest.mark.api()
class TestUpdateProperty(TestCase):
    DATABASE_ID = "1596567a3bbf80d58251f1159e5c40fa"

    def setUp(self) -> None:
        self.page = create_empty_page(database_id=self.DATABASE_ID)
        return super().setUp()

    def tearDown(self) -> None:
        remove_page(page_id=self.page.page_id)
        return super().setUp()

    def test_ファイルを変更する(self):
        self.skipTest("ファイルプロパティを作成するところから")



from lotion.block import Block
from lotion.block.rich_text.rich_text import RichText


class Code(Block):
    def __init__(  # noqa: PLR0913
        self,
        rich_text: RichText,
        language: str | None = None,
        id: str | None = None,  # noqa: A002
        archived: bool | None = None,
        created_time: str | None = None,
        last_edited_time: str | None = None,
        has_children: bool | None = None,
        parent: dict | None = None,
    ) -> None:
        super().__init__(
            id, archived, created_time, last_edited_time, has_children, parent
        )
        self.rich_text = rich_text
        self.language = language

    @staticmethod
    def of(block: dict) -> "Code":
        code = block["code"]
        rich_text = RichText.from_entity(code["rich_text"])
        language = code.get("language")
        return Code(
            rich_text=rich_text,
            language=language,
            id=block["id"],
            archived=block["archived"],
            created_time=block["created_time"],
            last_edited_time=block["last_edited_time"],
            has_children=block["has_children"],
            parent=block["parent"],
        )

    @staticmethod
    def from_plain_text(text: str, language: str | None = None) -> "Code":
        rich_text = RichText.from_plain_text(text)
        return Code(rich_text=rich_text, language=language)

    @property
    def type(self) -> str:
        return "code"

    def to_dict_sub(self) -> dict:
        return {
            "rich_text": self.rich_text.to_dict(),
            "language": self.language,
        }

    def to_slack_text(self) -> str:
        language = self.language or ""
        return f"```{language}\n{self.rich_text.to_plain_text()}\n```"



from unittest import TestCase

import pytest

from lotion import Lotion
from lotion.properties import Title


@pytest.mark.api()
class TestApiSelect(TestCase):
    DATABASE_ID = "15a6567a3bbf80b4a76fc106b37fb92f"

    def setUp(self) -> None:
        self.suite = Lotion.get_instance()

    def test_すべての選択肢を取得する(self):
        # Then
        actual = self.suite.fetch_all_selects(
            database_id=self.DATABASE_ID,
            name="セレクト",
        )

        # When: セレクトA、セレクトBの2つの選択肢があること
        self.assertEqual(actual.size, 2)
        self.assertIsNotNone(actual.get("セレクトA"))
        self.assertIsNotNone(actual.get("セレクトB"))

    def test_セレクトを更新する(self):
        # Given
        page = self.suite.create_page_in_database(
            database_id=self.DATABASE_ID, properties=[Title.from_plain_text(text="テスト")]
        )

        # When
        select_prop = self.suite.fetch_select(database_id=self.DATABASE_ID, name="セレクト", status_name="セレクトB")
        properties = page.properties.append_property(select_prop)
        self.suite.update_page(page_id=page.page_id.value, properties=properties.values)
        actual = self.suite.retrieve_page(page_id=page.page_id.value)

        # Then
        select = actual.get_select(name="セレクト")
        self.assertEqual(select.selected_name, "セレクトB")



from dataclasses import dataclass
from typing import Any

from lotion.filter.condition_ruleset import ConditionRuleset
from lotion.filter.condition import Prop, Cond


@dataclass(frozen=True)
class Builder:
    conditions: list[dict]

    @staticmethod
    def create() -> "Builder":
        return Builder(conditions=[])

    def add(self, prop_type: Prop, prop_name: str, cond_type: Cond, value: Any = None) -> "Builder":
        ConditionRuleset(prop_type, cond_type, value).validate()
        param = {
            "property": prop_name,
            prop_type.value: {
                cond_type.value: value if value is not None else {},
            },
        }
        return Builder(conditions=[*self.conditions, param])

    def is_empty(self) -> bool:
        return len(self.conditions) == 0

    def build(self) -> dict:
        if len(self.conditions) == 0:
            raise ValueError("Filter is empty")
        if len(self.conditions) == 1:
            return self.conditions[0]
        return {
            "and": self.conditions,
        }



from unittest import TestCase

import pytest

from lotion.filter.builder import Builder
from lotion.filter.condition import Prop, Cond
from lotion import Lotion


@pytest.mark.api()
class TestSearch(TestCase):
    DATABASE_ID = "15d6567a3bbf8032ada3e0a42892c357"

    def setUp(self) -> None:
        self.suite = Lotion.get_instance()
        return super().setUp()

    def test_検索_シンプルなテキスト検索(self):
        # Given
        filter_param = {
            "property": "テキスト",
            "rich_text": {
                "contains": "A",
            },
        }
        self._search_and_assert(filter_param, 1)

    def test_検索_複数の条件指定(self):
        # Given
        filter_param = (
            Builder.create()
            .add(Prop.RICH_TEXT, "名前", Cond.STARTS_WITH, "テスト")
            .add(Prop.NUMBER, "数値", Cond.GREATER_THAN, 50)
            .build()
        )
        self._search_and_assert(filter_param, 1)

    def test_検索_日付の検索(self):
        # Given
        filter_param = Builder.create().add(Prop.DATE, "日付", Cond.AFTER, "2021-01-01").build()
        self._search_and_assert(filter_param, 1)

    def test_検索_日付の検索_1年前以内(self):
        # Given
        filter_param = Builder.create().add(Prop.DATE, "日付", Cond.PAST_YEAR).build()
        self._search_and_assert(filter_param, 1)

    def test_or条件(self):
        # Given
        filter_param_a = Builder.create().add(Prop.RICH_TEXT, "名前", Cond.EQUALS, "テストA").build()
        filter_param_b = Builder.create().add(Prop.RICH_TEXT, "テキスト", Cond.EQUALS, "テキストB").build()
        filter_param = {
            "or": [filter_param_a, filter_param_b],
        }
        self._search_and_assert(filter_param, 2)

    def _search_and_assert(self, filter_param: dict, expected: int):
        # When
        actual = self.suite.retrieve_database(
            database_id=self.DATABASE_ID,
            filter_param=filter_param,
        )

        # Then
        self.assertEqual(expected, len(actual))



from unittest import TestCase

import pytest

from lotion.properties.status import Status
from test.test_lotion.lotion_utils import create_empty_page, remove_page, update_page


@pytest.mark.api()
class TestApiStatus(TestCase):
    DATABASE_ID = "15d6567a3bbf80c18391ec4c8d780e6a"

    def setUp(self) -> None:
        self.page = create_empty_page(database_id=self.DATABASE_ID)
        return super().setUp()

    def tearDown(self) -> None:
        remove_page(page_id=self.page.page_id)
        return super().setUp()

    def test_ステータスを変更する(self):
        status_prop = Status.from_status_name(name="ステータス", status_name="未着手")
        actual = update_page(page=self.page, property=status_prop)
        self.assertEqual(actual.get_status(name="ステータス").status_name, "未着手")



from unittest import TestCase

import pytest

from lotion.properties.url import Url
from test.test_lotion.lotion_utils import create_empty_page, remove_page, update_page


@pytest.mark.api()
class TestUpdateProperty(TestCase):
    DATABASE_ID = "15d6567a3bbf80b9b213fcd260e2358e"

    def setUp(self) -> None:
        self.page = create_empty_page(database_id=self.DATABASE_ID)
        return super().setUp()

    def tearDown(self) -> None:
        remove_page(page_id=self.page.page_id)
        return super().setUp()

    def test_URLを変更する(self):
        url_prop = Url.from_url(name="URL", url="https://example.com")
        actual = update_page(page=self.page, property=url_prop)
        self.assertEqual(actual.get_url(name="URL").url, "https://example.com")

        url_empty_prop = Url.empty(name="URL")
        actual = update_page(page=self.page, property=url_empty_prop)
        self.assertEqual(actual.get_url(name="URL").url, "")



from unittest import TestCase


from lotion.filter.builder import Builder
from lotion.filter.condition import Prop, Cond

# https://developers.notion.com/reference/post-database-query-filter


class TestBuilder(TestCase):
    def setUp(self) -> None:
        return super().setUp()

    def test_タイトルで絞りこむ(self):
        actual = (
            Builder.create()
            .add(prop_type=Prop.RICH_TEXT, prop_name="名前", cond_type=Cond.EQUALS, value="テストA")
            .build()
        )
        expected = {
            "property": "名前",
            "rich_text": {
                "equals": "テストA",
            },
        }
        self.assertEqual(expected, actual)



from os import name
from unittest import TestCase

import pytest

from lotion import Lotion
from lotion.properties import Title
from lotion.properties import Select


@pytest.mark.api()
class TestApiMultiSelect(TestCase):
    DATABASE_ID = "15c6567a3bbf80818512f43db108616f"

    def setUp(self) -> None:
        self.suite = Lotion.get_instance()

        select_a_prop = Select.of(
            name="セレクトA",
            param={
                "id": "oHJt",
                "type": "select",
                "select": {"id": "d9d95b7b-53e9-4497-8d8f-9aac9bb281ac", "name": "セレクトA", "color": "purple"},
            },
        )

    def test_すべての選択肢を取得する(self):
        # Then
        actual = self.suite.fetch_all_multi_select_elements(
            database_id=self.DATABASE_ID,
            name="マルチセレクト",
        )

        # When: A, B, C, Dの4つの選択肢がある
        print(actual)
        self.assertEqual(actual.size, 4)
        self.assertIsNotNone(actual.get("A"))
        self.assertIsNotNone(actual.get("B"))
        self.assertIsNotNone(actual.get("C"))
        self.assertIsNotNone(actual.get("D"))

    def test_セレクトを更新する(self):
        # Given
        page = self.suite.create_page_in_database(
            database_id=self.DATABASE_ID, properties=[Title.from_plain_text(text="テスト")]
        )

        # When
        multi_select_prop = self.suite.fetch_multi_select(
            database_id=self.DATABASE_ID, name="マルチセレクト", multi_select_names=["B", "D"]
        )
        properties = page.properties.append_property(multi_select_prop)
        self.suite.update_page(page_id=page.page_id.value, properties=properties.values)
        actual = self.suite.retrieve_page(page_id=page.page_id.value)

        # Then
        multi_select = actual.get_multi_select(name="マルチセレクト")
        multi_select_names = [value.name for value in multi_select.values]
        multi_select_names.sort()
        self.assertEqual(multi_select_names, ["B", "D"])



from unittest import TestCase

import pytest

from lotion.properties.checkbox import Checkbox
from test.test_lotion.lotion_utils import create_empty_page, remove_page, update_page


@pytest.mark.api()
class TestApiCheckbox(TestCase):
    DATABASE_ID = "15d6567a3bbf8075a116e8bbb4f76967"

    def setUp(self) -> None:
        self.page = create_empty_page(database_id=self.DATABASE_ID)
        return super().setUp()

    def tearDown(self) -> None:
        remove_page(page_id=self.page.page_id)
        return super().setUp()

    def test_チェックボックスを変更する(self):
        checkbox_prop = Checkbox.true(name="チェックボックス")
        actual = update_page(page=self.page, property=checkbox_prop)
        self.assertEqual(actual.get_checkbox(name="チェックボックス").checked, True)

        checkbox_false_prop = Checkbox.false(name="チェックボックス")
        actual = update_page(page=self.page, property=checkbox_false_prop)
        self.assertEqual(actual.get_checkbox(name="チェックボックス").checked, False)



from unittest import TestCase

import pytest

from lotion.properties.text import Text
from test.test_lotion.lotion_utils import create_empty_page, remove_page, update_page


@pytest.mark.api()
class TestApiText(TestCase):
    DATABASE_ID = "15d6567a3bbf80db8cb6d63ab1fecf22"

    def setUp(self) -> None:
        self.page = create_empty_page(database_id=self.DATABASE_ID)
        return super().setUp()

    def tearDown(self) -> None:
        remove_page(page_id=self.page.page_id)
        return super().setUp()

    def test_テキストを変更する(self):
        text_prop = Text.from_plain_text(name="テキスト", text="テスト")
        actual = update_page(page=self.page, property=text_prop)
        self.assertEqual(actual.get_text(name="テキスト").text, "テスト")

        text_empty_prop = Text.empty(name="テキスト")
        actual = update_page(page=self.page, property=text_empty_prop)
        self.assertEqual(actual.get_text(name="テキスト").text, "")



from dataclasses import dataclass

from lotion.properties.property import Property


@dataclass
class Number(Property):
    number: int | None
    type: str = "number"

    def __init__(
        self,
        name: str,
        id: str | None = None,  # noqa: A002
        number: int | None = None,
    ) -> None:
        self.name = name
        self.id = id
        self.number = number

    def add(self, count: int) -> "Number":
        prev = self.number if self.number is not None else 0
        return Number(
            name=self.name,
            id=self.id,
            number=prev + count,
        )

    @staticmethod
    def of(name: str, param: dict) -> "Number":
        if param["number"] is None:
            return Number(name=name, id=param["id"])
        return Number(
            name=name,
            id=param["id"],
            number=param["number"],
        )

    @staticmethod
    def empty(name: str) -> "Number":
        return Number(name=name)

    def is_empty(self) -> bool:
        return self.number is None

    @property
    def value(self) -> int:
        return self.number or 0

    def __dict__(self) -> dict:
        result = {
            "type": self.type,
            self.type: self.number,
        }
        if self.id is not None:
            result["id"] = self.id
        return {
            self.name: result,
        }

    @staticmethod
    def from_num(name: str, value: int) -> "Number":
        return Number(
            name=name,
            number=value,
        )

    def value_for_filter(self):
        return self.number



from lotion import Lotion
from lotion.base_page import BasePage
from lotion.properties.property import Property
from lotion.page import PageId


def create_empty_page(database_id: str):
    lotion = Lotion.get_instance()
    return lotion.create_page_in_database(database_id=database_id)


def update_page(page: BasePage, property: Property):
    # When
    lotion = Lotion.get_instance()
    properties = page.properties.append_property(property)
    lotion.update_page(page_id=page.page_id.value, properties=properties.values)

    # Then
    return lotion.retrieve_page(page_id=page.page_id.value)


def remove_page(page_id: PageId):
    lotion = Lotion.get_instance()
    lotion.remove_page(page_id.value)



from unittest import TestCase

import pytest

from lotion.properties.number import Number
from test.test_lotion.lotion_utils import create_empty_page, remove_page, update_page


@pytest.mark.api()
class TestApiNumber(TestCase):
    DATABASE_ID = "15d6567a3bbf80189707fd2234965886"

    def setUp(self) -> None:
        self.page = create_empty_page(database_id=self.DATABASE_ID)
        return super().setUp()

    def tearDown(self) -> None:
        remove_page(page_id=self.page.page_id)
        return super().setUp()

    def test_数値を変更する(self):
        number_prop = Number.from_num(name="数値", value=1)
        actual = update_page(page=self.page, property=number_prop)
        self.assertEqual(actual.get_number(name="数値").number, 1)

        number_empty_prop = Number.empty(name="数値")
        actual = update_page(page=self.page, property=number_empty_prop)
        self.assertEqual(actual.get_number(name="数値").number, None)



from dataclasses import dataclass

from lotion.properties.property import Property
from lotion.properties.title import Title


@dataclass(frozen=True)
class Properties:
    values: list[Property]

    def __post_init__(self) -> None:
        for value in self.values:
            if not isinstance(value, Property):
                msg = f"values must be Property. value: {value}"
                raise TypeError(msg)

    def __dict__(self) -> dict:
        result = {}
        for value in self.values:
            result = {**result, **value.__dict__()}
        return result

    @staticmethod
    def from_dict(properties: dict[str, dict]) -> "Properties":
        values = []
        for key, value in properties.items():
            values.append(Property.from_dict(key, value))
        return Properties(values=values)

    def append_property(self, prop: Property) -> "Properties":
        props = []

        is_updated = False
        for original_prop in self.values:
            if original_prop is None:
                # この場合があるらしい。どこかで混じったかな?
                continue
            if prop.name == original_prop.name:
                props.append(prop)
                is_updated = True
            else:
                props.append(original_prop)

        # valuesにない場合は新規で追加する
        if not is_updated:
            props.append(prop)

        return Properties(values=props)

    def get_title(self) -> Title:
        for value in self.values:
            if isinstance(value, Title):
                return value
        msg = f"Title property not found. properties: {self.values}"
        raise Exception(msg)

    def get_property(self, name: str, instance_class: type) -> Property | None:
        for value in self.values:
            if isinstance(value, instance_class) and value.name == name:
                return value
        return None

    def exclude_for_update(self) -> "Properties":
        """
        更新時にエラーとなるプロパティを除外する
        """
        exclude_types = ["button", "created_by", "last_edited_by", "formula", "rollup", "unique_id"]
        return Properties(values=[prop for prop in self.values if prop.type not in exclude_types])

    def is_empty(self) -> bool:
        return len(self.values) == 0



from lotion.block import Block


class ColumnList(Block):
    column_list: dict
    type: str = "column_list"

    def __init__(  # noqa: PLR0913
        self,
        column_list: dict,
        id: str | None = None,  # noqa: A002
        archived: bool | None = None,
        created_time: str | None = None,
        last_edited_time: str | None = None,
        has_children: bool | None = None,
        parent: dict | None = None,
    ) -> None:
        super().__init__(id, archived, has_children, created_time, last_edited_time, parent)
        self.column_list = column_list

    @staticmethod
    def of(block: dict) -> "ColumnList":
        column_list = block["column_list"]
        return ColumnList(
            id=block["id"],
            archived=block["archived"],
            created_time=block["created_time"],
            last_edited_time=block["last_edited_time"],
            has_children=block["has_children"],
            parent=block["parent"],
            column_list=column_list,
        )

    def to_dict_sub(self) -> dict:
        return {
            "column_list": self.column_list,
        }

    def to_slack_text(self) -> str:
        return ""



import sys
from unittest import TestCase

sys.path.append("notion_api")

# クラス名の一致をチェックするロジックがあるので、完全に合わせておく
from lotion.properties.properties import Properties
from lotion.properties.title import Title


class TestProperties(TestCase):
    def setUp(self) -> None:
        title = Title.from_plain_text(name="名前", text="タイトル")
        self.suite = Properties(values=[title])
        return super().setUp()

    def test_プロパティを置換する(self):
        # Given
        new_title = Title.from_plain_text(name="名前", text="新しいタイトル")

        # When
        properties = self.suite.append_property(new_title)

        # Then
        # print(properties.values)
        self.assertEqual(1, len(properties.values))
        self.assertEqual("新しいタイトル", properties.get_title().text)



import pytest


def pytest_configure(config):
    config.addinivalue_line("markers", "current: As current ones")
    config.addinivalue_line("markers", "learning: As learning exercises")
    config.addinivalue_line("markers", "api: As using the Notion API")
    config.addinivalue_line("markers", "slow: As slow ones")
    config.addinivalue_line("markers", "minimum: 最低限やっておきたいテスト")
    config.addinivalue_line("markers", "all: 全てのテスト. -m allで実行すると自動的に付与される")


def pytest_collection_modifyitems(config, items):
    """apiマークがついているテストを自動的に除外する。ただし、-mオプションで指定された場合は除外しない。"""
    selected_marker = config.getoption("-m")  # -mで指定されたマークを取得

    for item in items:
        # "api" がマークされているかどうかを確認
        if "api" in item.keywords:
            # -m で "api" が指定されていなければ、スキップマークを追加
            # if "api" not in selected_marker:
            if selected_marker is None or len(selected_marker) == 0:
                item.add_marker(pytest.mark.skip(reason="api マークがついているのでスキップ"))

        if "all" in selected_marker:
            item.add_marker(pytest.mark.all())



from unittest import TestCase

import pytest
from lotion.block import (
    Bookmark,
    BulletedListItem,
    Callout,
    Code,
    Divider,
    Embed,
    Heading,
    Image,
    NumberedListItem,
    Paragraph,
    Quote,
    ToDo,
    Video,
)
from lotion import Lotion


@pytest.mark.api()
class TestAppendBlock(TestCase):
    PAGE_ID = "1596567a3bbf8049803de1ffe3616d9e"

    def setUp(self):
        self.suite = Lotion.get_instance()
        self.suite.clear_page(self.PAGE_ID)

    def test_ブロックを追加する(self):
        blocks = []
        blocks.append(Paragraph.from_plain_text(text="テスト"))
        blocks.append(Bookmark.from_url(url="https://www.google.com/"))
        blocks.append(BulletedListItem.from_plain_text(text="テスト1"))
        blocks.append(BulletedListItem.from_plain_text(text="テスト2"))
        blocks.append(Divider())
        blocks.append(Embed.from_url(url="https://www.google.com/"))
        blocks.append(Heading.from_plain_text(heading_size=1, text="テスト"))
        blocks.append(Image.from_external_url(url="https://d3swar8tu7yuby.cloudfront.net/IMG_6286_thumb.jpg"))
        blocks.append(NumberedListItem.from_plain_text(text="テスト1"))
        blocks.append(NumberedListItem.from_plain_text(text="テスト2"))
        blocks.append(Quote.from_plain_text(text="テスト"))
        blocks.append(ToDo.from_plain_text(text="テスト1"))
        blocks.append(ToDo.from_plain_text(text="テスト2", checked=True))
        blocks.append(Video.from_external_url(url="https://www.youtube.com/watch?v=L5mF2uBKhS8"))
        blocks.append(Callout.from_plain_text(text="テスト"))
        blocks.append(Code.from_plain_text(text="print('hello')", language="python"))

        for i in range(len(blocks)):
            actual = self.suite.append_block(block_id=self.PAGE_ID, block=blocks[i])
            self.assertTrue("results" in actual)

        page = self.suite.retrieve_page(self.PAGE_ID)
        for i in range(len(blocks)):
            self.assertIsInstance(page.block_children[i], blocks[i].__class__)



from typing import Any

from lotion.properties import (
    PhoneNumber,
    CreatedBy,
    Email,
    LastEditedBy,
    Button,
    Checkbox,
    CreatedTime,
    Date,
    LastEditedTime,
    MultiSelect,
    Number,
    Properties,
    Property,
    Relation,
    Rollup,
    Select,
    Status,
    Text,
    Title,
    Url,
    People,
    Formula,
    UniqueId,
)


class PropertyTranslator:
    @staticmethod
    def from_dict(properties: dict[str, dict]) -> Properties:
        values = []
        for key, value in properties.items():
            values.append(PropertyTranslator.from_property_dict(key, value))
        return Properties(values=[value for value in values if value is not None])

    @staticmethod
    def from_property_dict(key: str, property_: dict[str, Any]) -> "Property":  # noqa: PLR0911
        type_ = property_["type"]
        match type_:
            case "title":
                return Title.from_property(key, property_)
            case "rich_text":
                return Text.from_dict(key, property_)
            case "multi_select":
                return MultiSelect.of(key, property_)
            case "select":
                return Select.of(key, property_)
            case "number":
                return Number.of(key, property_)
            case "checkbox":
                return Checkbox.of(key, property_)
            case "date":
                return Date.of(key, property_)
            case "status":
                return Status.of(key, property_)
            case "url":
                return Url.of(key, property_)
            case "relation":
                return Relation.of(key, property_)
            case "last_edited_time":
                return LastEditedTime.create(key, property_["last_edited_time"])
            case "created_time":
                return CreatedTime.create(key, property_["created_time"])
            case "rollup":
                return Rollup.of(key, property_)
            case "button":
                return Button.of(key, property_)
            case "people":
                return People.of(key, property_)
            case "email":
                return Email.of(key, property_)
            case "phone_number":
                return PhoneNumber.of(key, property_)
            case "created_by":
                return CreatedBy.of(key, property_)
            case "last_edited_by":
                return LastEditedBy.of(key, property_)
            case "formula":
                return Formula.of(key, property_)
            case "unique_id":
                return UniqueId.of(key, property_)
            case _:
                msg = f"Unsupported property type: {type_} {property_}"
                raise Exception(msg)



from dataclasses import dataclass

from lotion.properties.property import Property


@dataclass
class Formula(Property):
    """Formula class

    ex.
    {'id': 'h_pG', 'type': 'formula', 'formula': {'type': 'number', 'number': 50}}
    """

    _formula: dict

    def __init__(
        self,
        name: str,
        formula: dict = {},
        id: str | None = None,  # noqa: A002
    ) -> None:
        self.name = name
        self._formula = formula
        self.id = id

    @staticmethod
    def of(key: str, param: dict) -> "Formula":
        return Formula(
            id=param["id"],
            name=key,
            formula=param["formula"],
        )

    @property
    def value(self) -> dict:
        formula_type = self._formula["type"]
        return self._formula[formula_type]

    @property
    def type(self) -> str:
        return "formula"

    def __dict__(self) -> dict:
        raise NotImplementedError("this dict method must not be called")

    def value_for_filter(self) -> str:
        raise NotImplementedError



from unittest import TestCase

import pytest

from lotion import Lotion
from lotion.page import PageId
from lotion.properties import Relation


@pytest.mark.api()
class TestApiRelationAndRollup(TestCase):
    DATABASE_ID = "15d6567a3bbf804e942dc49d808bf73a"

    def setUp(self) -> None:
        self.suite = Lotion.get_instance()

    @pytest.mark.minimum()
    def test_リレーションを変更する(self):
        page = self.suite.create_page_in_database(database_id=self.DATABASE_ID)

        # Given
        page_id = PageId("15d6567a3bbf804191c4e0dbd42644fe")
        relation_prop = Relation.from_id(name="リレーション", id=page_id.value)

        # When, Then
        properties = page.properties.append_property(relation_prop)
        self.suite.update_page(page_id=page.page_id.value, properties=properties.values)
        actual = self.suite.retrieve_page(page_id=page.page_id.value)
        actual_relation = actual.get_relation(name="リレーション")
        self.assertEqual(actual_relation.id_list, [page_id.value])

    def test_ロールアップを取得する(self):
        # When
        page_id = "15d6567a3bbf80c98c16d9b846d73946"
        page = self.suite.retrieve_page(page_id=page_id)

        # Then: 2が取得できること
        actual = page.get_rollup(name="ロールアップ_計算")
        self.assertEqual(actual.value, 2)

        # Then: 【消さない】リレーションA、【消さない】リレーションBが取得できること
        actual = page.get_rollup(name="ロールアップ_文字列")
        title_elements = [v["title"][0] for v in actual.value]
        text_contents = [t["text"]["content"] for t in title_elements]
        import json

        print(json.dumps(text_contents, indent=2, ensure_ascii=False))
        self.assertEqual(text_contents, ["【消さない】リレーションA", "【消さない】リレーションB"])



from dataclasses import dataclass
from typing import Any

from lotion.properties.property import Property


@dataclass
class LastEditedBy(Property):
    def __init__(
        self,
        name: str,
        last_edited_by: dict,
        id: str | None = None,  # noqa: A002
    ) -> None:
        self.name = name
        self.last_edited_by = last_edited_by
        self.id = id

    @staticmethod
    def of(key: str, param: dict) -> "LastEditedBy":
        return LastEditedBy(id=param["id"], name=key, last_edited_by=param["last_edited_by"])

    def __dict__(self) -> dict[str, Any]:
        return {
            self.name: {
                "id": self.id,
                "type": self.type,
                "last_edited_by": self.last_edited_by,
            },
        }

    @property
    def type(self) -> str:
        return "last_edited_by"  # NOTE: created_timeではなくdateにする

    def value_for_filter(self) -> str:
        raise NotImplementedError()
